// SPDX-FileCopyrightText: 2023 Carl Schwan <carl@carlschwan.eu>
// SPDX-License-Identifier: LGPL-2.0-or-later

#pragma once

#include <KJob>
#include <KMime/Message>

class EwsItem;

namespace EwsMailFactory
{
QString create(const KMime::Message::Ptr &message);
};
